import logging

from collections import OrderedDict
from rest_framework.fields import *
from rest_framework import serializers as r_s

logger = logging.getLogger(__name__)

DRF_COMPOSITE_FIELDS = ["ListSerializer", "ManyRelatedField"]

DRF_LIST_FIELDS = ["ListSerializer", "ManyRelatedField"]

DRF_COMPOSITE_FIELD_RELATION_NAMES = {"ListSerializer": "child", "ManyRelatedField": "child_relation"}

DRF_FIELDS = {
        BooleanField: "boolean",
        NullBooleanField: "boolean",

        CharField: "string",
        EmailField: "string",
        RegexField: "string",
        SlugField: "string",
        URLField: "string",
        UUIDField: "string",
        FilePathField: "string",
        IPAddressField: "string",

        IntegerField: "integer",
        FloatField: "number",
        DecimalField: "number",

        DateTimeField: "string",
        DateField: "string",
        TimeField: "string",
        DurationField: "string",

        ChoiceField: "string",
        MultipleChoiceField: "string",

        FileField: "string",
        ImageField: "string",

        ListField: "list",
        DictField: "dict",

        JSONField: "json",

        r_s.PrimaryKeyRelatedField: "integer",
        r_s.ManyRelatedField: "list"
    }


class BaseAdapter:

    def __init__(self, serializer):
        self.serializer = serializer
        self.instance = serializer()

    def transform_field(self, field):
        return {
            "read_only": False,
            "write_only": False,
            "required": True,
        }

    def get_serializer_fields(self):
        return OrderedDict([])

    def get_transformed_data(self):
        fields = self.get_serializer_fields()
        nested = []
        data = OrderedDict([])

        for field in fields.items():
            logger.critical(field[0])
            logger.critical(field[1])
            logger.critical("=====")
            field_data = self.transform_field(field)
            data[field[0]] = field_data[0]
            nested += field_data[1]
        if hasattr(self.serializer.Meta, 'model'):
            name = self.serializer.Meta.model.__name__
        else:
            name = self.serializer.__name__
        nested.append({
            'name': name,
            'fields': data
        })
        return nested


class SerializerAdapter(BaseAdapter):
    def get_serializer_fields(self):
        return self.instance.get_fields()

    def transform_field(self, field):
        nested = []
        data = {
            'name': field[0],
            'read_only': field[1].read_only,
            'write_only': field[1].write_only,
            'required': field[1].required,
            'nullable': field[1].allow_null,
        }
        field_class = field[1].__class__
        type_data = DRF_FIELDS.get(field_class, None)
        data['openapi_type'] = type_data
        data['type'] = field_class.__name__
        if data['type'] in DRF_COMPOSITE_FIELDS:
            child_class = getattr(field[1], DRF_COMPOSITE_FIELD_RELATION_NAMES[data["type"]]).__class__
            if child_class.__name__ == "PrimaryKeyRelatedField":
                data['child'] = 'number'
            else:
                nested_data = SerializerAdapter(child_class).get_transformed_data()
                data['child'] = nested_data[0]["name"]
                nested.extend(nested_data)
        return data, nested



import logging
from django.conf import settings
from .adapters import *

logger = logging.getLogger(__name__)


class BaseCompiler:
    adapter_class = None

    def treverse_urls(self, root_pattern):
        patterns = []
        for pattern in root_pattern:
            try:
                patterns += self.treverse_urls(pattern.url_patterns)
            except:
                patterns.append(pattern)
        return patterns

    @staticmethod
    def apiview_filter(view_callback):
        if hasattr(view_callback, "view_class"):
            if hasattr(view_callback.view_class, "serializer_class"):
                return True
        return False

    def get_all_serializer_data(self, additional_serializers=[]):
        root_urlconf = __import__(settings.ROOT_URLCONF)
        all_urlpatterns = root_urlconf.urls.urlpatterns
        url_patterns = self.treverse_urls(all_urlpatterns)
        serializers = [
            p.callback.view_class.serializer_class
            for p in url_patterns
            if self.apiview_filter(p.callback)
        ] + additional_serializers
        data = [
            self.adapter_class(serializer).get_transformed_data()
            for serializer in serializers
        ]
        data = [val for sublist in data for val in sublist]
        return data

    def generate_field(self, data):
        return "Not Implemented"

    def generate_serializer_body(self, serializer_data):
        return "Not Implemented"

    def generate(self, additional_serializers=[]):
        return "Not Implemented"


class TypeScriptCompiler(BaseCompiler):
    adapter_class = SerializerAdapter

    def generate_field(self, data):
        if data["type"] in DRF_LIST_FIELDS:
            dtype = f"Array<{data['child']}>"
        else:
            dtype = data["openapi_type"]

        if data["nullable"]:
            dtype = f"{dtype} | null"
        return f"\t{'readonly ' if data['read_only'] else ''}{data['name']}{'?' if not data['required'] and not data['read_only'] else ''}: {dtype};"

    def generate_serializer_body(self, serializer_data):
        return "\n".join(
            [
                f"export interface {serializer_data['name']} {{",
                *[
                    self.generate_field(field)
                    for field in serializer_data["fields"].values()
                ],
                "}",
            ]
        )

    def generate(self, additional_serializers=[]):
        data = self.get_all_serializer_data(additional_serializers)
        interface = [self.generate_serializer_body(sbody) for sbody in data]
        return "\n\n".join(interface)
